<?php

/* Icinga DB Web | (c) 2021 Icinga GmbH | GPLv2 */

namespace Icinga\Module\Icingadb\Command\Transport;

use Icinga\Exception\IcingaException;

/**
 * Exception thrown if a command was not sent
 */
class CommandTransportException extends IcingaException
{
}
