// HostGroup- and -ServiceGroupGrid styles

ul.item-table.group-grid {
  grid-template-columns: repeat(auto-fit, 15em);
  grid-gap: 1em 2em;

  .table-row {
    margin: -.25em;
    padding: .25em;
    border-radius: .5em;
  }

  li.group-grid-cell {
    .title {
      align-items: center;
    }

    .visual {
      margin-right: 1em;
    }

    .content {
      line-height: 1;

      a {
        display: inline-block;
        max-width: 10em;
        text-align: center;
      }
    }

    .state-badge {
      width: 2.5em;
      height: 2.5em;
      line-height: 2;
    }
  }
}

.content.full-width ul.item-table.group-grid {
  margin: 0 1em;
}
