// Style

// Layout

.comment-list:not(.detailed) .list-item {
  .title > i:first-child {
    margin-right: 0;
  }

  .title > .subject + .badge,
  .title > .badge + .subject,
  .title > .badge:last-of-type {
    margin-left: 0;
  }

  .title a {
    &:not(.subject) {
      .text-ellipsis();
    }
  }

  .title .subject:not(:last-child) {
    margin-left: 0;
  }

  .title .subject:nth-child(3):last-child {
    margin-left: 0;
  }
}

.comment-list.minimal .list-item {
  .user-ball {
    font-size: .857em;
    height: 1.75em;
    line-height: 1.5em;
    width: 1.75em;
  }
}

.comment-list.detailed .list-item {
  .title > .subject:nth-child(3),
  .title > .badge + .subject:last-child {
    margin-left: .3em;
  }

  .caption {
    max-height: 4.5em;
    white-space: normal;
  }
}
