<?php

/* Icinga DB Web | (c) 2020 Icinga GmbH | GPLv2 */

namespace Icinga\Module\Icingadb\Common;

class Icons
{
    const COMMENT = 'comment';

    const HOST_DOWN = 'sitemap';

    const IN_DOWNTIME = 'plug';

    const IS_ACKNOWLEDGED = 'check';

    const IS_FLAPPING = 'bolt';

    const IS_PERSISTENT = 'thumbtack';

    const NOTIFICATION = 'bell';

    const REMOVE = 'trash';

    const USER = 'user';

    const USERGROUP = 'users';

    const WARNING = 'exclamation-triangle';
}
