<?php

/* Icinga DB Web | (c) 2020 Icinga GmbH | GPLv2 */

namespace Icinga\Module\Icingadb\Widget\ItemList;

use Icinga\Module\Icingadb\Common\ListItemCommonLayout;

class CommentListItem extends BaseCommentListItem
{
    use ListItemCommonLayout;
}
