// Layout

.controls .user-list,
.controls .usergroup-list {
  .usergroup-ball,
  .user-ball {
    height: 1.5em;
    width: 1.5em;
    line-height: ~"calc(1.5em - 4px)";
    display: block;
  }

  .title br {
    display: none;
  }

  .list-item {
    & > .visual {
      padding-top: 0.25em;
    }

    & > .col {
      padding: .25em 0;
    }
  }
}
